# -*-coding:utf-8 -*-
"""
:创建时间: 2023/5/2 7:53
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division

if False:
    from typing import *

import cpmel.cmds as cc
from rig_core.ctx import Ctx
from rig_operation_graph._utils import *

is_load, _, _ = cc.pluginInfo('quatNodes', q=True, set=True)
if not is_load:
    print("loading quatNodes.mll")
    cc.loadPlugin('quatNodes', qt=True)


def quaternion_product(ctx, left, right):
    n = ctx.create_node('quatProd')
    n.rename('quaternion_product')
    set_or_connect_quaternion(left, n.attr('input1Quat'))
    set_or_connect_quaternion(right, n.attr('input2Quat'))
    return n.attr('outputQuat')


def quaternion_add(ctx, left, right):
    n = ctx.create_node('quatAdd')
    n.rename('quaternion_add')
    set_or_connect_quaternion(left, n.attr('input1Quat'))
    set_or_connect_quaternion(right, n.attr('input2Quat'))
    return n.attr('outputQuat')


def quaternion_sub(ctx, left, right):
    n = ctx.create_node('quatSub')
    n.rename('quaternion_sub')
    set_or_connect_quaternion(left, n.attr('input1Quat'))
    set_or_connect_quaternion(right, n.attr('input2Quat'))
    return n.attr('outputQuat')


def quaternion_invert(ctx, value):
    n = ctx.create_node('quatInvert')
    n.rename('quaternion_invert')
    set_or_connect_quaternion(value, n.attr('inputQuat'))
    return n.attr('outputQuat')


def quaternion_normalize(ctx, value):
    n = ctx.create_node('quatNormalize')
    n.rename('quaternion_normalize')
    set_or_connect_quaternion(value, n.attr('inputQuat'))
    return n.attr('outputQuat')


def quaternion_conjugate(ctx, value):
    n = ctx.create_node('quatConjugate')
    n.rename('quaternion_conjugate')
    set_or_connect_quaternion(value, n.attr('inputQuat'))
    return n.attr('outputQuat')


def quaternion_to_euler(ctx, value):
    n = ctx.create_node('quatToEuler')
    n.rename('quaternion_to_euler')
    set_or_connect_quaternion(value, n.attr('inputQuat'))
    return n.attr('outputRotate')


def quaternion_from_euler(ctx, value):
    n = ctx.create_node('eulerToQuat')
    n.rename('quaternion_from_euler')
    set_or_connect_3d(value, n.attr('inputRotate'))
    return n.attr('outputQuat')


__all__ = [
    'quaternion_product',
    'quaternion_add',
    'quaternion_sub',
    'quaternion_invert',
    'quaternion_normalize',
    'quaternion_conjugate',
    'quaternion_to_euler',
    'quaternion_from_euler',
]

if __name__ == '__main__':
    from maya_test_tools import question_open_maya_gui

    ctx = Ctx()

    # Test quaternion_product
    out_attr = quaternion_product(ctx, (0, 0, 0, 1), (0, 0, 0, 1))
    print("quaternion_product:", out_attr.get_value())

    # Test quaternion_add
    out_attr = quaternion_add(ctx, (0, 0, 0, 1), (0, 0, 0, 1))
    print("quaternion_add:", out_attr.get_value())

    # Test quaternion_sub
    out_attr = quaternion_sub(ctx, (0, 0, 0, 1), (0, 0, 0, 1))
    print("quaternion_sub:", out_attr.get_value())

    # Test quaternion_invert
    out_attr = quaternion_invert(ctx, (0, 0, 0, 1))
    print("quaternion_invert:", out_attr.get_value())

    # Test quaternion_normalize
    out_attr = quaternion_normalize(ctx, (0, 0, 0, 1))
    print("quaternion_normalize:", out_attr.get_value())

    # Test quaternion_conjugate
    out_attr = quaternion_conjugate(ctx, (0, 0, 0, 1))
    print("quaternion_conjugate:", out_attr.get_value())

    # Test quaternion_to_euler
    out_attr = quaternion_to_euler(ctx, (0, 0, 0, 1))
    print("quaternion_to_euler:", out_attr.get_value())

    # Test quaternion_from_euler
    out_attr = quaternion_from_euler(ctx, (0, 0, 0))
    print("quaternion_from_euler:", out_attr.get_value())

    question_open_maya_gui()



from cpmel.cmds import Attr
from rig_core.all import *
from rig_operation_graph._utils import *

from typing import Union

InValue = Union[float, int, Attr]
OutAttr = Attr

__all__ = ['float_add', 'float_sub', 'float_mul', 'float_div',
           'float_pow', 'float_mean', 'float_max', 'float_min', 'float_sum',
           'float_sin', 'float_cos', 'float_tan']


def float_add(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_sub(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_mul(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_div(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_pow(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_mean(ctx: Ctx, *values: InValue) -> OutAttr: pass


def float_max(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_min(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def float_sum(ctx: Ctx, *values: InValue) -> OutAttr: pass


def float_sin(ctx: Ctx, value: InValue) -> OutAttr: pass


def float_cos(ctx: Ctx, value: InValue) -> OutAttr: pass


def float_tan(ctx: Ctx, value: InValue) -> OutAttr: pass



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/29 5:30
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_operation_graph._utils import *

__all__ = ['float_add', 'float_sub', 'float_mul', 'float_div',
           'float_pow', 'float_mean', 'float_max', 'float_min', 'float_sum',
           'float_sin', 'float_cos', 'float_tan']


def float_add(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('float_add')
    n['operation'] = 1
    set_or_connect(a, n.input1D[0])
    set_or_connect(b, n.input1D[1])
    return n.output1D


def float_sub(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('float_sub')
    n['operation'] = 2
    set_or_connect(a, n.input1D[0])
    set_or_connect(b, n.input1D[1])
    return n.output1D


def float_mul(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('float_mul')
    n['operation'] = 1
    set_or_connect(a, n.input1X)
    set_or_connect(b, n.input2X)
    return n.outputX


def float_div(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('float_div')
    n['operation'] = 2
    set_or_connect(a, n.input1X)
    set_or_connect(b, n.input2X)
    return n.outputX


def float_pow(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('float_pow')
    n['operation'] = 3
    set_or_connect(a, n.input1X)
    set_or_connect(b, n.input2X)
    return n.outputX


def float_mean(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('float_mean')
    n['operation'] = 3
    for id_, v in enumerate(values):
        set_or_connect(v, n.input1D[id_])
    return n.output1D


def float_max(ctx, a, b):
    n = ctx.create_node('condition')
    n.rename('float_max')
    n['operation'] = 2
    set_or_connect(a, n.firstTerm)
    set_or_connect(b, n.secondTerm)
    set_or_connect(a, n.colorIfTrueR)
    set_or_connect(b, n.colorIfFalseR)

    return n.outColorR


def float_min(ctx, a, b):
    n = ctx.create_node('condition')
    n.rename('float_min')
    n['operation'] = 4
    set_or_connect(a, n.firstTerm)
    set_or_connect(b, n.secondTerm)
    set_or_connect(a, n.colorIfTrueR)
    set_or_connect(b, n.colorIfFalseR)

    return n.outColorR


def float_sum(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('float_sum')
    n['operation'] = 1
    for id_, v in enumerate(values):
        set_or_connect(v, n.input1D[id_])
    return n.output1D


def float_sin(ctx, value):
    exp = cc.expression(s='', o='', ae=False, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_sin')
    input_attr = ctx.add_attribute(exp, 'input_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = sin({})".format(output_attr.name(), input_attr.name())
    )
    set_or_connect(value, input_attr)
    return output_attr


def float_cos(ctx, value):
    exp = cc.expression(s='', o='', ae=False, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_cos')
    input_attr = ctx.add_attribute(exp, 'input_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = cos({})".format(output_attr.name(), input_attr.name())
    )
    set_or_connect(value, input_attr)
    return output_attr


def float_tan(ctx, value):
    exp = cc.expression(s='', o='', ae=False, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_tan')
    input_attr = ctx.add_attribute(exp, 'input_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = tan({})".format(output_attr.name(), input_attr.name())
    )
    set_or_connect(value, input_attr)
    return output_attr


if __name__ == "__main__":
    from rig_core.ctx import Ctx
    from maya_test_tools import question_open_maya_gui

    c = Ctx()

    out_attr = float_min(c, 0, 1)
    print(out_attr, out_attr.get_value())

    out_attr = float_max(c, 0, 1)
    print(out_attr, out_attr.get_value())

    out_attr = float_mean(c, 0, 1)
    print(out_attr, out_attr.get_value())

    out_attr = float_pow(c, 10, 10)
    print(out_attr, out_attr.get_value())

    out_attr = float_sin(c, 0.5)
    print(out_attr, out_attr.get_value())
    out_attr = float_cos(c, 0.5)
    print(out_attr, out_attr.get_value())
    out_attr = float_tan(c, 0.5)
    print(out_attr, out_attr.get_value())

    question_open_maya_gui()



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/22 10:13
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc

__all__ = ['current_frame']


def current_frame():
    """
    :rtype: cc.Attr
    """
    return cc.new_object('time1.outTime')



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/12 21:47
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

混合计算模块
"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

from rig_operation_graph._utils import *

from rig_core.all import *

if False:
    from typing import AnyStr, Any, List, Tuple, Dict

__all__ = ['blend']


def blend(ctx, a, b, control):
    """
    混合两个属性

    :type ctx: Ctx
    :type a: cc.Attr or float
    :type b: cc.Attr or float
    :type control: cc.Attr or float
    :rtype: cc.Attr
    """
    node = cc.createNode('blendTwoAttr')
    set_or_connect(a, node.attr('input[0]'))
    set_or_connect(b, node.attr('input[1]'))
    set_or_connect(control, node.attributesBlender)
    return node.output


if __name__ == '__main__':
    def test():
        from maya_test_tools import open_file, question_open_maya_gui

        ctx = Ctx()

        print('blend', blend(ctx, 2, 1, 0.5).value)

        question_open_maya_gui()


    test()



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/11 21:52
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division

from rig_operation_graph.drive import (
    range as range_drive
)

from rig_operation_graph.matrix import (
    new_matrix_from_transform_attr,
    new_matrix_from_transform_value,
    copy_matrix as matrix_copy,
    mult_matrix as matrix_mult,
    inverse_matrix as matrix_inverse,
    to_transform_attr as matrix_to_transform_attr,
)
from rig_operation_graph.matrix import *

from rig_operation_graph.clamp import clamp
from rig_operation_graph.reverse import reverse
from rig_operation_graph.blend import blend
from rig_operation_graph.distance import distance

from rig_operation_graph.logic import *
from rig_operation_graph.math import *
from rig_operation_graph.math3d import *
from rig_operation_graph.time import *
from rig_operation_graph.random import *

from rig_operation_graph.additional import *

from rig_operation_graph.quaternion import *